﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.51
@EndOfDesignText@
Sub Class_Globals
	Public Points As List
	Type InternalBCPathColumnData (MinY As Float, MaxY As Float, StartPoint As Float, EndPoint As Float, SignY As Float, LastY As Float)
	Type InternalBCPathPointData (X As Float, Y As Float, ColumnData As InternalBCPathColumnData)
	Type BCBrush (Source As BitmapCreator, SrcOffsetX As Int, SrcOffsetY As Int, IsColorSource As Boolean, BlendAll As Boolean, _
		BlendBorders As Boolean, ColorPM As PremultipliedColor)
	Type InternalCanvasDrawTask (Brush As BCBrush, Code As Int, Values() As Float, Filled As Boolean)
	Private DataReady As Boolean
	Private Bounds As B4XRect
	Public InternalCounterClockWise As Boolean
	Public DrawConnectionSegments As Boolean = True
End Sub

'Initializes the path and sets the first point.
Public Sub Initialize (X As Float, Y As Float) As BCPath
	Points.Initialize
	Return Reset (X, Y)
End Sub

'Resets the path.
Public Sub Reset (X As Float, Y As Float) As BCPath
	Points.Clear
	Points.Add(CreatePoint(X, Y))
	DataReady = False
	Return Me
End Sub

Private Sub CreatePoint(x As Float, y As Float) As InternalBCPathPointData
	Dim d As InternalBCPathPointData
	d.X = x
	d.Y = y
	Return d
End Sub

'Adds a line to the given point.
Public Sub LineTo (X As Float, Y As Float) As BCPath
	Dim NewPoint As InternalBCPathPointData = CreatePoint(X, Y)
	Points.Add(NewPoint)
	DataReady = False
	Return Me
End Sub

Private Sub BuildColumnData
	If Points.Size < 1 Then Return
	Dim First As InternalBCPathPointData
	Dim Start, EndI, StepI As Int
	If InternalCounterClockWise Then
		Start = 1
		EndI = Points.Size - 1
		StepI = 1
		First  = Points.Get(0)
	Else
		First = Points.Get(Points.Size - 1)
		Start = Points.Size - 2
		EndI = 0
		StepI = -1
	End If
	
	First.ColumnData = Null
	For i = Start To EndI Step StepI
		Dim Second As InternalBCPathPointData = Points.Get(i)
		Dim dy As Float = Second.Y - First.Y
		If First.ColumnData = Null Or ((First.ColumnData.SignY >= 0) <> (dy >= 0)) Then
			Second.ColumnData = CreateNewColumnData(First.Y, dy)
		Else
			Second.ColumnData = First.ColumnData
		End If
		If dy > 0 Then
			Second.ColumnData.MaxY = Second.Y
		Else if dy < 0 Then
			Second.ColumnData.MinY = Second.y
		End If
		First = Second
	Next
End Sub

Private Sub FindOrder
	Dim size As Int = Points.Size
	Dim sum As Float
	For i = 0 To size - 1
		Dim CurrentPoint As InternalBCPathPointData = Points.Get(i)
		Dim NextPoint As InternalBCPathPointData = Points.Get((i + 1) Mod size)
		sum = sum + (NextPoint.X - CurrentPoint.X) * (NextPoint.Y + CurrentPoint.Y)
	Next
	InternalCounterClockWise = sum >= 0
End Sub

Private Sub CreateNewColumnData (y As Float, signy As Float) As InternalBCPathColumnData
	Dim CurrentColumn As InternalBCPathColumnData
	CurrentColumn.MinY = Y
	CurrentColumn.MaxY = Y
	CurrentColumn.LastY = Y
	CurrentColumn.SignY = signy
	Return CurrentColumn
End Sub

'Internal sub.
Public Sub PrepareForFilling
	If DataReady Then Return
	Dim lp As InternalBCPathPointData = Points.Get(Points.Size - 1)
	Dim fp As InternalBCPathPointData = Points.Get(0)
	If lp.x <> fp.X Or lp.Y <> fp.Y Then
		LineTo(fp.X, fp.Y)
	End If
	FindOrder
	BuildColumnData
	FindBoundingRect
	DataReady = True
End Sub

'Removes the last point.
Public Sub RemoveLastPoint
	Points.RemoveAt(Points.Size - 1)
	DataReady = False
End Sub

'Call Invalidate if you modifed the Points list directly.
Public Sub Invalidate
	DataReady = False
End Sub

'Returns the path bounding rectangle.
Public Sub FindBoundingRect As B4XRect
	If DataReady Then Return Bounds
	Dim MinX = 0x7fffffff, MaxX = 0x80000000, MinY = 0x7fffffff, MaxY = 0x80000000 As Float
	For Each p As InternalBCPathPointData In Points
		Dim x As Float = p.X
		Dim y As Float = p.Y
		MinX = Min(MinX, x)
		MinY = Min(MinY, y)
		MaxX = Max(MaxX, x)
		MaxY = Max(MaxY, y)
	Next
	Dim Bounds As B4XRect
	Bounds.Initialize(MinX, MinY, MaxX, MaxY)
	Return Bounds
End Sub

'Creates a copy of the path. This is used by BitmapCreator.AsyncDrawPath.
Public Sub Clone As BCPath
	Dim NewPath As BCPath
	NewPath.Initialize(0, 0)
	NewPath.Points.Clear
	NewPath.Points.AddAll(Points)
	NewPath.DataReady = DataReady
	NewPath.Bounds = Bounds
	NewPath.DrawConnectionSegments = DrawConnectionSegments
	Return NewPath
End Sub